from model.sense import Sense
from model.editable import Editable
from model.tags import import_label_list

class Entry(Editable):
    def __init__(self, entry_xml):
        status = entry_xml.querySelector("head status")
        headword = entry_xml.querySelector("head headword lemma")
        grammar = entry_xml.querySelector("head grammar category")
        comment = entry_xml.querySelector("head comment")
        
        self.status = status.textContent if status else ""
        self.headword = headword.textContent if headword else ""
        self.grammar = grammar.textContent if grammar else ""
        self.comment = comment.textContent if comment else ""
        self.variants = [v.textContent for v in entry_xml.querySelectorAll("head variantList variant")]
        self.related_entries = [re.textContent for re in entry_xml.querySelectorAll("head relatedEntryList relatedEntry")]
        
        self.lexical_unit = {}
        lex_unit = entry_xml.querySelector("lexical_unit lexeme,lexicalUnit lexeme")
        if lex_unit:
            self.lexical_unit['id'] = lex_unit.getAttribute("lexical_unit_lexeme_id")
            self.lexical_unit['text'] = lex_unit.textContent
        
        self.measure = {}
        measure = entry_xml.querySelector("measureList measure")
        if measure:
            self.measure["source"] = measure.getAttribute("source")
            self.measure["type"] = measure.getAttribute("type")
            self.measure["text"] = measure.textContent

        self.labels = import_label_list("head labelList label", entry_xml)
        
        self.senses = [Sense(sense_xml, i) for i, sense_xml in 
                       enumerate(entry_xml.querySelectorAll("body senseList sense"))]
    
    def get_measure_text(self):
        return self.measure["text"] if "text" in self.measure else ""
    
    def remove_translation(self, translation):
        for sense in self.senses:
            for cluster in sense.translations:
                if translation in cluster:
                    cluster.remove(translation)
                    return
            for example in sense.examples:
                for cluster in example.translations:
                    if translation in cluster:
                        cluster.remove(translation)
                        return
                    
